//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcoigdt.hpp,v 1.13 2004/02/18 12:18:01 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcoigdt.hpp,v $
// $Revision: 1.13 $
//--------------------------------------------------------------------
// The internal implementation of various Roma gadgets.
//--------------------------------------------------------------------

#ifndef __KXCOIGDT_HPP__
#define __KXCOIGDT_HPP__

#include "kxcotrb.hpp"
#include "kxcoutil.hpp"
#include "kxcosec.hpp"

class RomaImplContext;


//--------------------------------------------------------------------
// Macro implementation of the assignment operator of a public object
// with a shareable implementation class.
//--------------------------------------------------------------------
#define KXCO_ASSIGN_SHARED(src)     \
    if (m_impl != src.m_impl) {     \
        m_impl->releaseInstance();  \
        m_impl = src.m_impl;        \
        m_impl->lockInstance(); }   \
    return *this;


//--------------------------------------------------------------------
// The RomaImplShareable class.
// This is the base class for an implementation which instance can be
// shared between multiply objects.
//--------------------------------------------------------------------
class KXCODLL RomaImplShareable
{
public:
    RomaImplShareable() : m_lockCount(1) {}
    void lockInstance();
    void releaseInstance();

protected:
    virtual ~RomaImplShareable();

private:
    RomaImplShareable(const RomaImplShareable&);
    RomaImplShareable& operator = (const RomaImplShareable&);

    // Reference counter.
    unsigned m_lockCount;
};


//--------------------------------------------------------------------
// The RomaImplDirAccess class.
// This is the internal implementation of the RomaDirectoryAccess
// public interface.
//--------------------------------------------------------------------
class KXCODLL RomaImplDirAccess : public RomaTraceable
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaImplDirAccess();

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    ~RomaImplDirAccess();

    //----------------------------------------------------------------
    // Copy and assignment.
    //----------------------------------------------------------------
    RomaImplDirAccess(const RomaImplDirAccess&);
    RomaImplDirAccess& operator = (const RomaImplDirAccess&);

    //----------------------------------------------------------------
    // Operator casting the object to a pointer to the
    // KXCDIRACCESSOPT structure.
    //----------------------------------------------------------------
    operator PKXCDIRACCESSOPT() const { 
        return (KXCDIRACCESSOPT*)&m_data; }

    //----------------------------------------------------------------
    // Set/get an user ID.
    //----------------------------------------------------------------
    bool setUserId(const char*);
    const char* getUserId() const { 
        return (const char*)m_data.Id; }

    //----------------------------------------------------------------
    // Set/get a password.
    //----------------------------------------------------------------
    bool setPassword(const char*);
    const char* getPassword() const { 
        return (const char*)m_data.Password; }

    //----------------------------------------------------------------
    // Output trace information.
    //----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;

private:
    //----------------------------------------------------------------
    // Directory access options.
    //----------------------------------------------------------------
    KXCDIRACCESSOPT m_data;
};


//--------------------------------------------------------------------
// The RomaImplMsgId class.
// This is the internal implementation of the RomaMsgId public
// interface.
//--------------------------------------------------------------------
class KXCODLL RomaImplMsgId : public RomaTraceable,
                              public RomaImplShareable
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaImplMsgId(const void* data = NULL);

    //----------------------------------------------------------------
    // Copy and assignment.
    //----------------------------------------------------------------
    RomaImplMsgId(const RomaImplMsgId&);
    RomaImplMsgId& operator = (const RomaImplMsgId&);

    //----------------------------------------------------------------
    // Comparison operators.
    //----------------------------------------------------------------
    bool operator == (const RomaImplMsgId&) const;
    bool operator != (const RomaImplMsgId&) const;

    //----------------------------------------------------------------
    // Operator casting the object to the "const char*" pointer.
    //----------------------------------------------------------------
    operator const char*() const { return m_msgIdPtr; }

    //----------------------------------------------------------------
    // Fill the whole Message ID data buffer with zeros.
    //----------------------------------------------------------------
    void setNull();

    //----------------------------------------------------------------
    // Set Message ID data.
    //----------------------------------------------------------------
    void set(const void*, int = KXCMSG_ID_LENGTH);

    //----------------------------------------------------------------
    // Copy Message ID data to the specified buffer.
    //----------------------------------------------------------------
    void copy(void*, int = Roma::ID_LENGTH) const;

    //----------------------------------------------------------------
    // Setup the object to refer to an external Message ID data to
    // avoid data copy.
    //----------------------------------------------------------------
    void useExternal(PKXCCHAR id) { m_msgIdPtr = (char*)id; }

    //----------------------------------------------------------------
    // Output trace information.
    //----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;

private:
    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    ~RomaImplMsgId() {}

    //----------------------------------------------------------------
    // Pointer to a Message ID data buffer. May point to an external
    // Message ID data.
    //----------------------------------------------------------------
    char* m_msgIdPtr;

    //----------------------------------------------------------------
    // Message ID data buffer.
    //----------------------------------------------------------------
    char m_msgId[Roma::ID_LENGTH];

    friend class KxcoStreamer;
};


//--------------------------------------------------------------------
// The RomaImplILSData class.
// This is the internal implementation of the RomaILSData public
// interface.
//--------------------------------------------------------------------
class KXCODLL RomaImplILSData : public RomaImplShareable, 
                                public RomaTraceable
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaImplILSData();

    //----------------------------------------------------------------
    // Constructor creating the object from the ILS data handle.
    //----------------------------------------------------------------
    RomaImplILSData(HKXCILS, RomaImplContext*, HKXC);

    //----------------------------------------------------------------
    // Set ILS data.
    //----------------------------------------------------------------
    bool setBuffer(void*, int, int, Roma::ILSType);

    //----------------------------------------------------------------
    // Get the first ILS data buffer.
    //----------------------------------------------------------------
    const void* getBuffer(int&, int, Roma::ILSType);

    //----------------------------------------------------------------
    // Get the next ILS data buffer.
    //----------------------------------------------------------------
    const void* getNextBuffer(int&, int&, Roma::ILSType&);

    //----------------------------------------------------------------
    // Indicates whether ILS data may be replaced.
    //----------------------------------------------------------------
    void setReplace(bool);

    //----------------------------------------------------------------
    // Operator casting the object to the ILS data handle (HKXCILS).
    //----------------------------------------------------------------
    operator HKXCILS() const { return m_dataSet; }

    //----------------------------------------------------------------
    // Output trace information.
    //----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;

private:
    //----------------------------------------------------------------
    // ILS data access options.
    //----------------------------------------------------------------
    KXCILSOPT m_options;

    //----------------------------------------------------------------
    // ILS data handle.
    //----------------------------------------------------------------
    HKXCILS m_dataSet;

    //----------------------------------------------------------------
    // Source context.
    //----------------------------------------------------------------
    RomaImplContext* m_context;
    HKXC m_bufOwner;

    //----------------------------------------------------------------
    // Private destructor.
    // The object should be deleted using the releaseInstance() method
    // only.
    //----------------------------------------------------------------
    virtual ~RomaImplILSData();

    //----------------------------------------------------------------
    // NOT IMPLEMENTED.
    //----------------------------------------------------------------
    RomaImplILSData(const RomaImplILSData&);
    RomaImplILSData& operator = (const RomaImplILSData&);

    friend class KxcoStreamer;
};


//--------------------------------------------------------------------
// The RomaImplLogInfo class.
// This is the internal implementation of the RomaLogInformation
// public interface.
//--------------------------------------------------------------------
class KXCODLL RomaImplLogInfo : public RomaTraceable
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaImplLogInfo();

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    ~RomaImplLogInfo();

    //----------------------------------------------------------------
    // Operator casting the object to a pointer to the KXCLOGDATA
    // structure.
    //----------------------------------------------------------------
    operator PKXCLOGDATA() const { return (KXCLOGDATA*)&m_data; }

    //----------------------------------------------------------------
    // Set the information severity.
    //----------------------------------------------------------------
    void setSeverity(Roma::Severity sev) { m_data.Severity = sev; }

    //----------------------------------------------------------------
    // Set the information format.
    //----------------------------------------------------------------
    void setFormat(KXCULONG format) { m_data.Format = format; }

    //----------------------------------------------------------------
    // Set user text.
    //----------------------------------------------------------------
    bool setUserText(const char*);

    //----------------------------------------------------------------
    // Set user data.
    //----------------------------------------------------------------
    bool setUserData(const void*, int);

    //----------------------------------------------------------------
    // Output trace information.
    //----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;

private:
    //----------------------------------------------------------------
    // Log data.
    //----------------------------------------------------------------
    KXCLOGDATA m_data;

    //----------------------------------------------------------------
    // NOT IMPLEMENTED.
    //----------------------------------------------------------------
    RomaImplLogInfo(const RomaImplLogInfo&);
    RomaImplLogInfo& operator = (const RomaImplLogInfo&);

    friend class KxcoStreamer;
};


//--------------------------------------------------------------------
// The RomaImplSyncpoint class.
// This is the internal implementation of the RomaSyncpointDef
// public interface.
//--------------------------------------------------------------------
class KXCODLL RomaImplSyncpoint : public RomaTraceable
{
public:
    //----------------------------------------------------------------
    // Constructors.
    //----------------------------------------------------------------
    RomaImplSyncpoint(Roma::SyncType);
    RomaImplSyncpoint(PKXCVOID);

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    ~RomaImplSyncpoint() {}

    //----------------------------------------------------------------
    // Copy and assignment will be generated by a compiler.
    //----------------------------------------------------------------

    //----------------------------------------------------------------
    // Set the syncpoint type.
    //----------------------------------------------------------------
    void setType(Roma::SyncType type) { m_type = type; }

    //----------------------------------------------------------------
    // Set the transaction context.
    //----------------------------------------------------------------
    void setContext(PKXCVOID context) { m_context = context; }

    //----------------------------------------------------------------
    // Fill the KXCSYNC with object data.
    //----------------------------------------------------------------
    void populate(KXCSYNC&) const;

    //----------------------------------------------------------------
    // Output trace information.
    //----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;

private:
    //----------------------------------------------------------------
    // Type of the syncpoint.
    //----------------------------------------------------------------
    KXCULONG m_type;

    //----------------------------------------------------------------
    // Transaction context of the syncpoint.
    //----------------------------------------------------------------
    PKXCVOID m_context;

    friend class KxcoStreamer;
};


//--------------------------------------------------------------------
// The RomaImplTheme class.
// This is the internal implementation of the RomaTheme public
// interface.
//--------------------------------------------------------------------
class KXCODLL RomaImplTheme : public RomaTraceable
{
public:
    //----------------------------------------------------------------
    // Constructor creating the object from the theme handle.
    //----------------------------------------------------------------
    RomaImplTheme(HKXCTHEME);

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    ~RomaImplTheme() {}

    //----------------------------------------------------------------
    // Copy and assignment will be generated by a compiler.
    //----------------------------------------------------------------

    //----------------------------------------------------------------
    // Assignment operator allowing to set a theme handle to the
    // object.
    //----------------------------------------------------------------
    RomaImplTheme& operator = (HKXCTHEME);

    //----------------------------------------------------------------
    // Operator casting the object to the theme handle.
    //----------------------------------------------------------------
    operator HKXCTHEME() const { return m_handle; }

    //----------------------------------------------------------------
    // Comparison operators.
    //----------------------------------------------------------------
    bool operator == (const RomaImplTheme&) const;
    bool operator != (const RomaImplTheme&) const;

    //----------------------------------------------------------------
    // Indicate whether the theme is NULL.
    //----------------------------------------------------------------
    bool isNull() const { return (m_handle == NULL_HKXCTHEME); }

    //----------------------------------------------------------------
    // Set the theme to NULL.
    //----------------------------------------------------------------
    void setNull() { m_handle = NULL_HKXCTHEME; }

    //----------------------------------------------------------------
    // Output trace information.
    //----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;

private:
    //----------------------------------------------------------------
    // Theme handle.
    //----------------------------------------------------------------
    HKXCTHEME m_handle;

    friend class KxcoStreamer;
};


//--------------------------------------------------------------------
// The RomaImplTimestamp class.
// This is the internal implementation of the RomaTimestamp public
// interface.
//--------------------------------------------------------------------
class KXCODLL RomaImplTimestamp
{
public:
    //----------------------------------------------------------------
    // Costructor initializing the object with seconds and
    // milliseconds.
    //----------------------------------------------------------------
    RomaImplTimestamp(KXCULONG, KXCULONG);
    RomaImplTimestamp(const KXCTIMESTAMP&);

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    ~RomaImplTimestamp() {}

    //----------------------------------------------------------------
    // Copy and assignment will be generated by a compiler.
    //----------------------------------------------------------------

    //----------------------------------------------------------------
    // Assignment operator allowing to setup the object with the
    // KXCTIMESTAMP stucture.
    //----------------------------------------------------------------
    RomaImplTimestamp& operator = (const KXCTIMESTAMP&);

    //----------------------------------------------------------------
    // Comparison operators.
    //----------------------------------------------------------------
    bool operator == (const RomaImplTimestamp&) const;
    bool operator != (const RomaImplTimestamp&) const;
    bool operator < (const RomaImplTimestamp&) const;
    bool operator <= (const RomaImplTimestamp&) const;
    bool operator > (const RomaImplTimestamp&) const;
    bool operator >= (const RomaImplTimestamp&) const;

    //----------------------------------------------------------------
    // Set object data to the KXCTIMESTAMP structure.
    //----------------------------------------------------------------
    void populate(KXCTIMESTAMP&) const;

    //----------------------------------------------------------------
    // Set/get seconds.
    //----------------------------------------------------------------
    KXCULONG getSeconds() const { return m_sec; }
    void setSeconds(KXCULONG val) { m_sec = val; }

    //----------------------------------------------------------------
    // Set/get milliseconds.
    //----------------------------------------------------------------
    KXCULONG getMilliseconds() const { return m_msec; }
    void setMilliseconds(KXCULONG val) { m_msec = val; }

private:
    //----------------------------------------------------------------
    // Seconds.
    //----------------------------------------------------------------
    KXCULONG m_sec;

    //----------------------------------------------------------------
    // Milliseconds.
    //----------------------------------------------------------------
    KXCULONG m_msec;

    friend class KxcoStreamer;
};


//--------------------------------------------------------------------
// The RomaImplSubject class.
// This is the internal implementation of the RomaSubject public
// interface.
//--------------------------------------------------------------------
class KXCODLL RomaImplSubject : public RomaTraceable
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaImplSubject();

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    ~RomaImplSubject() {}

    //----------------------------------------------------------------
    // Copy and assignment will be generated by a compiler.
    //----------------------------------------------------------------

    //----------------------------------------------------------------
    // Get/set a subject style.
    //----------------------------------------------------------------
    void setStyle(Roma::Style style) { m_style = style; }
    int getStyle() { return m_style; }

    //----------------------------------------------------------------
    // Get/set a subject name.
    //----------------------------------------------------------------
    void setName(const char*);
    const char* getName() const { return (*m_name ? m_name : NULL); }

    //----------------------------------------------------------------
    // Fill the KXCSUBJECT structure with object data.
    //----------------------------------------------------------------
    void populate(KXCSUBJECT&) const;

    //----------------------------------------------------------------
    // Output trace information.
    //----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;

private:
    //----------------------------------------------------------------
    // Subject style.
    //----------------------------------------------------------------
    int m_style;

    //----------------------------------------------------------------
    // Subject name (null-terminated).
    //----------------------------------------------------------------
    char m_name[KXCSUBJECT_NAME_LENGTH + 1];

    friend class KxcoStreamer;
};


//--------------------------------------------------------------------
// The RomaImplSecItem class.
// This class represents a security definition item.
//--------------------------------------------------------------------
class RomaImplSecItem 
{
public:
    //----------------------------------------------------------------
    // Constructor initializing the item with data.
    //----------------------------------------------------------------
    RomaImplSecItem(int, const void*, int);

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    ~RomaImplSecItem();

    //----------------------------------------------------------------
    // Set/replace item data.
    //----------------------------------------------------------------
    bool replaceData(const void*, int);

    //----------------------------------------------------------------
    // Target ILS identifier.
    //----------------------------------------------------------------
    int m_method;

    //----------------------------------------------------------------
    // Data length.
    //----------------------------------------------------------------
    int m_length;

    //----------------------------------------------------------------
    // Data pointer.
    //----------------------------------------------------------------
    void* m_data;
};


//--------------------------------------------------------------------
// The linked list of security definition items.
//--------------------------------------------------------------------
KLINKED_LIST_OF(RomaImplSecItem)


//--------------------------------------------------------------------
// The RomaImplSecurity class.
// This is the internal implementation of the RomaSecurityDef public
// interface.
//--------------------------------------------------------------------
class KXCODLL RomaImplSecurity : public RomaImplShareable,
                                 public RomaTraceable
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaImplSecurity();

    //----------------------------------------------------------------
    // Operator casting the object to a pointer to the KXCSECDEF
    // structure.
    //----------------------------------------------------------------
    operator PKXCSECDEF() const;

    //----------------------------------------------------------------
    // Add/replace a security definition item.
    //----------------------------------------------------------------
    bool setBuffer(int, const void*, int);

    //----------------------------------------------------------------
    // Setup the specified security definition with the C API
    // representation of object data.
    //----------------------------------------------------------------
    bool populate(RomaSecurityDef&) const;

    //----------------------------------------------------------------
    // Output trace information.
    //----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;

private:
    //----------------------------------------------------------------
    // Linked list of security definition items.
    //----------------------------------------------------------------
    RomaImplSecItemLList m_list;

    //----------------------------------------------------------------
    // Pointer to the KXCSECDEF structure containing object data in
    // the C API representation.
    //----------------------------------------------------------------
    KXCSECDEF* m_pSecDef;

    //----------------------------------------------------------------
    // Indicate whether the object contains data in the C API
    // representation only.
    //----------------------------------------------------------------
    bool m_cloned;

    //----------------------------------------------------------------
    // Size of object data in the C API representation.
    //----------------------------------------------------------------
    unsigned m_total;

    //----------------------------------------------------------------
    // Create the C API representation of object data.
    //----------------------------------------------------------------
    bool consolidate(RomaImplSecurity&) const;

    //----------------------------------------------------------------
    // Remove object data.
    //----------------------------------------------------------------
    void clear();

    //----------------------------------------------------------------
    // Private destructor.
    // The object should be deleted using the releaseInstance() method
    // only.
    //----------------------------------------------------------------
    virtual ~RomaImplSecurity();

    //----------------------------------------------------------------
    // NOT IMPLEMENTED.
    //----------------------------------------------------------------
    RomaImplSecurity(const RomaImplSecurity&);
    RomaImplSecurity& operator = (const RomaImplSecurity&);

    friend class KxcoStreamer;
};


#endif // __KXCOIGDT_HPP__

