//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcostqa.cpp,v 1.8 2003/02/21 22:48:09 emueh Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcostqa.cpp,v $
// $Revision: 1.8 $
//--------------------------------------------------------------------
// The RomaStorageQueue public interface.
//--------------------------------------------------------------------

#include "kxcointa.hpp"
#include "kxcoicxt.hpp"
#include "kxcoipxy.hpp"
#include "kxcostq.hpp"

TRACE_MODULE_VERSION($Revision: 1.8 $);

//--------------------------------------------------------------------
// RTTI definition.
//--------------------------------------------------------------------
ROMA_DEFINE_TYPEID(RomaStorageQueue);

//--------------------------------------------------------------------
// Default constructor.
// Creates the object that should be initialized with open() method.
// For internal use only.
//--------------------------------------------------------------------
RomaStorageQueue::RomaStorageQueue()
{
    TRACE_ENTRY(RomaStorageQueue::RomaStorageQueue, << TRACE(this));

    m_impl = new RomaImplStorageQueue;

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Constructor.
// Opens the queue using an established connection of the RomaClient.
//--------------------------------------------------------------------
RomaStorageQueue::RomaStorageQueue(RomaClient& client,
                                   const char* name,
                                   KXCULONG options,
                                   const char* model)
{
    TRACE_ENTRY(RomaStorageQueue::RomaStorageQueue, << TRACE(this));

    m_impl = new RomaImplStorageQueue;
    m_impl->open(RomaImplProxy::getImpl(client), name, options, model);

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Constructor.
// Opens the queue using an established connection of the
// RomaBusinessElement.
//--------------------------------------------------------------------
RomaStorageQueue::RomaStorageQueue(RomaBusinessElement& be,
                                   const char* name,
                                   KXCULONG options,
                                   const char* model)
{
    TRACE_ENTRY(RomaStorageQueue::RomaStorageQueue, << TRACE(this));

    m_impl = new RomaImplStorageQueue;
    m_impl->open(RomaImplProxy::getImpl(be), name, options, model);

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Destructor.
// Closes the queue.
//--------------------------------------------------------------------
RomaStorageQueue::~RomaStorageQueue()
{
    TRACE_ENTRY(RomaStorageQueue::~RomaStorageQueue, << TRACE(this));

    delete m_impl;

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Open the queue using an established connection of the RomaClient.
//--------------------------------------------------------------------
bool RomaStorageQueue::open(RomaClient& client, 
                            const char* name,
                            KXCULONG options,
                            const char* model)
{
    TRACE_ENTRY(RomaStorageQueue::open, << TRACE(this));

    bool success = m_impl->open(RomaImplProxy::getImpl(client), name,
        options, model);

    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Open the queue using an established connection of the
// RomaBusinessElement.
//--------------------------------------------------------------------
bool RomaStorageQueue::open(RomaBusinessElement& be,
                            const char* name,
                            KXCULONG options,
                            const char* model)
{
    TRACE_ENTRY(RomaStorageQueue::open, << TRACE(this));

    bool success = m_impl->open(RomaImplProxy::getImpl(be), name, 
        options, model);

    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Close the queue.
//--------------------------------------------------------------------
bool RomaStorageQueue::close()
{
    TRACE_ENTRY(RomaStorageQueue::close, << TRACE(this));

    bool success = m_impl->close();

    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Store a message on the queue.
// It will call marshal() method before put the message to the queue.
//--------------------------------------------------------------------
bool RomaStorageQueue::store(RomaStorable& storable,
                             const RomaStoreOptions& options)
{
    TRACE_ENTRY(RomaStorageQueue::store, << TRACE(this));

    bool success = m_impl->store(storable, options);

    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Retrieve a message from the queue.
// Message data will be stored into the specified buffer.
// The createStorable() method will be invoked to perform a message
// unmarshaling.
//--------------------------------------------------------------------
RomaStorable* 
RomaStorageQueue::retrieve(const RomaRetrieveOptions& options,
                           void* buffer, int& length)
{
    TRACE_ENTRY(RomaStorageQueue::retrieve, << TRACE(this));

    RomaStorable* result = 
        m_impl->retrieve(*this, buffer, length, options);

    TRACE_INT_RETURN(result);
}

//--------------------------------------------------------------------
// Retrieve a message from the queue.
// The createStorable() method will be invoked to perform a message
// unmarshaling.
//--------------------------------------------------------------------
RomaStorable* 
RomaStorageQueue::retrieve(const RomaRetrieveOptions& options)
{
    TRACE_ENTRY(RomaStorageQueue::retrieve, << TRACE(this));

    RomaStorable* result = m_impl->retrieve(*this, options);

    TRACE_INT_RETURN(result);
}

//--------------------------------------------------------------------
// An application can override this method to perform a message
// unmarshaling. The default implementation does nothing.
//--------------------------------------------------------------------
bool RomaStorageQueue::createStorable(
    const RomaStorable& baseStorable, RomaStorable*& newStorable)
{
    // Both messages will share common implementation object.
    newStorable = RomaImplProxy::createPublic(baseStorable);
    return true;
}

//--------------------------------------------------------------------
// Returns true if the queue is open.
//--------------------------------------------------------------------
bool RomaStorageQueue::isOpen() const
{
    return m_impl->isOpen();
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaStorageQueue::trace(RomaTraceStream& s) const
{
    s << TRACE_CLASS(RomaStorageQueue, << TRACE(m_impl)) << "->";
    m_impl->trace(s);
}

