//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : RomaStorageQueue.java
//
// Defines      : class RomaStorageQueue
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;

import com.candle.roma.kxc.raw.*;
import java.io.*;

/**
* The RomaStorageQueue class enables the storing and retrieval
* of message from a Roma storage queue.
*/
public class RomaStorageQueue
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999,2000,2001";
  
  private RomaContext m_ctx;
  private String      m_name;       //  Name of storage queue
  private int         m_opts;       //  Queue open options
  private long        m_hStore = 0; //  Handle
  
  /**
  * Construct and open the named storage queue using a given client.
  *
  * @param context  should specify a valid and connected Roma Client
  * @param name     name of the storage queue to open
  * @param options  open options. Either the constant OPEN_DEFAULT
  *                 or a combination of the following flag
  *                 constants :-
  * <ul>
  *   <li>OPEN_NON_XACTIONAL
  *   <li>OPEN_EXCLUSIVE
  *   <li>OPEN_DELETE_ON_CLOSE
  *   <li>OPEN_MUST_EXIST
  *   <li>OPEN_OUTPUT
  *   <li>OPEN_INPUT
  *   <li>OPEN_PEEK
  * </ul>
  * @param modelQName model queue name
  * 
  * @exception  RomaError  if the queue open fails.
  */
  public RomaStorageQueue( RomaClient client,
                           String     name,
                           int        opts,
                           String     modelQName )
    throws RomaError
  {
    RomaContext.kxcOper( 
            client.kxcCtx().openStorageQ( name, opts, modelQName ) );

    m_hStore = client.kxcCtx().hStore();
    m_ctx    = client;
    m_name   = name;
    m_opts   = opts;
  }
  
  /**
  * Construct and open the named storage queue using a given client.
  *
  * @param context  should specify a valid and connected Roma Client
  * @param name     name of the storage queue to open
  * @param options  open options. Either the constant OPEN_DEFAULT
  *                 or a combination of the following flag
  *                 constants :-
  * <ul>
  *   <li>OPEN_NON_XACTIONAL
  *   <li>OPEN_EXCLUSIVE
  *   <li>OPEN_DELETE_ON_CLOSE
  *   <li>OPEN_MUST_EXIST
  *   <li>OPEN_OUTPUT
  *   <li>OPEN_INPUT
  *   <li>OPEN_PEEK
  * </ul>
  * 
  * @exception  RomaError  if the queue open fails.
  */
  public RomaStorageQueue( RomaClient client,
                           String     name,
                           int        opts )
    throws RomaError
  {
    RomaContext.kxcOper( 
            client.kxcCtx().openStorageQ( name, opts, null ) );

    m_hStore = client.kxcCtx().hStore();
    m_ctx    = client;
    m_name   = name;
    m_opts   = opts;
  }
  
  /**
  * Construct and open the named storage queue using a given client.
  * Use OPEN_DEFAULT option.
  *
  * @param context  should specify a valid and connected Roma Client
  * @param name     name of the storage queue to open
	 *
   * @exception  RomaError  if the queue open fails.
  */
  public RomaStorageQueue( RomaClient client,
                           String     name )
    throws RomaError
  {
    RomaContext.kxcOper( 
            client.kxcCtx().openStorageQ( name, Roma.OPEN_DEFAULT, null ) );
    
    m_hStore = client.kxcCtx().hStore();
    m_ctx    = client;
    m_name   = name;
    m_opts   = Roma.OPEN_DEFAULT;
  }
  
  /**
  * Contruct and open the named storage queue using a given
  * business element.
  *
  * @param be  should specify a valid and connected Roma Business Element
  * @param name  name of the storage queue to open
  * @param options  open options. Either the constant OPEN_DEFAULT
  *                 or a combination of the following flag
  *                 constants :-
  * <ul>
  *   <li>OPEN_NON_XACTIONAL
  *   <li>OPEN_EXCLUSIVE
  *   <li>OPEN_DELETE_ON_CLOSE
  *   <li>OPEN_MUST_EXIST
  *   <li>OPEN_OUTPUT
  *   <li>OPEN_INPUT
  *   <li>OPEN_PEEK
  * </ul>
  * @param modelQName model queue name
  *
  * @exception  RomaError  if the queue open fails.
  */
  public RomaStorageQueue( RomaBusinessElement  be,
                           String               name,
                           int                  opts,
                           String               modelQName )
    throws RomaError
  {
    RomaContext.kxcOper( 
            be.kxcCtx().openStorageQ( name, opts, modelQName) );

    m_hStore = be.kxcCtx().hStore();
    m_ctx    = be;
    m_name   = name;
    m_opts   = opts;
  }
  
  /**
  * Contruct and open the named storage queue using a given
  * business element.
  *
  * @param be  should specify a valid and connected Roma Business Element
  * @param name  name of the storage queue to open
  * @param options  open options. Either the constant OPEN_DEFAULT
  *                 or a combination of the following flag
  *                 constants :-
  * <ul>
  *   <li>OPEN_NON_XACTIONAL
  *   <li>OPEN_EXCLUSIVE
  *   <li>OPEN_DELETE_ON_CLOSE
  *   <li>OPEN_MUST_EXIST
  *   <li>OPEN_OUTPUT
  *   <li>OPEN_INPUT
  *   <li>OPEN_PEEK
  * </ul>
  *
  * @exception  RomaError  if the queue open fails.
  */
  public RomaStorageQueue( RomaBusinessElement  be,
                           String               name,
                           int                  opts )
    throws RomaError
  {
    RomaContext.kxcOper( 
            be.kxcCtx().openStorageQ( name, opts, null ) );

    m_hStore = be.kxcCtx().hStore();
    m_ctx    = be;
    m_name   = name;
    m_opts   = opts;
  }
  
  /**
  * Contruct and open the named storage queue using a given
  * business element. Use OPEN_DEFAULT option.
  *
  * @param be  should specify a valid Roma and connected Business Element
  * @param name  name of the storage queue to open
  *
  * @exception  RomaError  if the queue open fails.
  */
  public RomaStorageQueue( RomaBusinessElement  be,
                           String               name )
    throws RomaError
  {
    RomaContext.kxcOper(
            be.kxcCtx().openStorageQ( name, Roma.OPEN_DEFAULT, null ) );

    m_hStore = be.kxcCtx().hStore();
    m_ctx    = be;
    m_name   = name;
    m_opts   = Roma.OPEN_DEFAULT;
  }
  
  /**
  * Open the named storage queue using a given client.
  *
  * @param client   should specify a valid and connected Roma Client
  * @param name     name of the storage queue to open
  * @param options  open options. Either the constant OPEN_DEFAULT
  *                 or a combination of the following flag
  *                 constants :-
  * <ul>
  *   <li>OPEN_NON_XACTIONAL
  *   <li>OPEN_EXCLUSIVE
  *   <li>OPEN_DELETE_ON_CLOSE
  *   <li>OPEN_MUST_EXIST
  *   <li>OPEN_OUTPUT
  *   <li>OPEN_INPUT
  *   <li>OPEN_PEEK
  * </ul>
  * @param modelQName model queue name
  *
  * @exception  RomaError  if the queue open fails.
  */
  public void open( RomaClient  client,
                    String      name,
                    int         opts,
                    String      modelQName )
    throws RomaError
  {
    close();

    RomaContext.kxcOper(
            client.kxcCtx().openStorageQ( name, opts, modelQName ) );

    m_hStore = client.kxcCtx().hStore();
    m_ctx    = client;
    m_name   = name;
    m_opts   = opts;
  }
  
  /**
  * Open the named storage queue using a given client.
  *
  * @param client   should specify a valid and connected Roma Client
  * @param name     name of the storage queue to open
  * @param options  open options. Either the constant OPEN_DEFAULT
  *                 or a combination of the following flag
  *                 constants :-
  * <ul>
  *   <li>OPEN_NON_XACTIONAL
  *   <li>OPEN_EXCLUSIVE
  *   <li>OPEN_DELETE_ON_CLOSE
  *   <li>OPEN_MUST_EXIST
  *   <li>OPEN_OUTPUT
  *   <li>OPEN_INPUT
  *   <li>OPEN_PEEK
  * </ul>
  *
  * @exception  RomaError  if the queue open fails.
  */
  public void open( RomaClient  client,
                    String      name,
                    int         opts )
    throws RomaError
  {
    open ( client, name, opts, null );
  }
  
  /**
  * Open the named storage queue using a given client.
  * Use OPEN_DEFAULT option.
  *
  * @param client should specify a valid and connected Roma Client
  * @param name name of the storage queue to open
  *
  * @exception  RomaError  if the queue open fails.
  */
  public void open( RomaClient  client,
                    String      name )
    throws RomaError
  {
    open ( client, name, Roma.OPEN_DEFAULT, null );
  }

  /**
  * Open the named storage queue using a given
  * business element.
  *
  * @param be  should specify a valid and connected Roma Business Element
  * @param name  name of the storage queue to open
  * @param options  open options. Either the constant OPEN_DEFAULT
  *                 or a combination of the following flag
  *                 constants :-
  * <ul>
  *   <li>OPEN_NON_XACTIONAL
  *   <li>OPEN_EXCLUSIVE
  *   <li>OPEN_DELETE_ON_CLOSE
  *   <li>OPEN_MUST_EXIST
  *   <li>OPEN_OUTPUT
  *   <li>OPEN_INPUT
  *   <li>OPEN_PEEK
  * </ul>
  * @param modelQName model queue name
  *
  * @exception  RomaError  if the queue open fails.
  */
  public void open( RomaBusinessElement be,
                    String              name,
                    int                 opts,
                    String              modelQName )
    throws RomaError
  {
    close();

    RomaContext.kxcOper( 
            be.kxcCtx().openStorageQ( name, opts, modelQName ) );

    m_hStore = be.kxcCtx().hStore();
    m_ctx    = be;
    m_name   = name;
    m_opts   = opts;
  }
  
  /**
  * Open the named storage queue using a given
  * business element.
  *
  * @param be  should specify a valid and connected Roma Business Element
  * @param name  name of the storage queue to open
  * @param options  open options. Either the constant OPEN_DEFAULT
  *                 or a combination of the following flag
  *                 constants :-
  * <ul>
  *   <li>OPEN_NON_XACTIONAL
  *   <li>OPEN_EXCLUSIVE
  *   <li>OPEN_DELETE_ON_CLOSE
  *   <li>OPEN_MUST_EXIST
  *   <li>OPEN_OUTPUT
  *   <li>OPEN_INPUT
  *   <li>OPEN_PEEK
  * </ul>
  *
  * @exception  RomaError  if the queue open fails.
  */
  public void open( RomaBusinessElement be,
                    String              name,
                    int                 opts )
    throws RomaError
  {
    open( be, name, opts, null);
  }
  
  /**
  * Open the named storage queue using a given
  * business element. Use OPEN_DEFAULT option.
  *
  * @param be  should specify a valid and connected Roma Business Element
  * @param name  name of the storage queue to open
  * @exception  RomaError  if the queue open fails.
  */
  public void open( RomaBusinessElement be,
                    String              name )
    throws RomaError
  {
    open( be, name, Roma.OPEN_DEFAULT, null);
  }
		
  /**
  * Close the storage queue.
  *
  * @exception RomaError  if the close queue call fails.
  */
  public void close()
    throws RomaError
  {
    try {
      if ( m_hStore != 0 )
      {
        RomaContext.kxcOper( 
          m_ctx.kxcCtx().closeStorageQ( m_hStore ) );
        m_hStore = 0;
      }
    }
    finally {
      m_hStore = 0;
    }
  }
  
  /**
  * Store a message on the storage queue according to the 
  * defaulted options.
  *
  * @param msg  message to put on the storage queue.
  *
  * @exception RomaError  thrown if the store operation
  *                           fails.
  */
  public void store( RomaStorable msg )
    throws RomaError
  {
    RomaStoreOptions opts = new RomaStoreOptions( ); 	
    store( msg, opts );
  }	
		
  /**
  * Store a message on the storage queue according to the 
  * supplied options.
  *
  * @param msg  message to put on the storage queue.
  * @param opt  options for the store action.
  *
  * @exception RomaError  thrown if the store operation
  *                           fails.
  */
  public void store( RomaStorable     msg,
                     RomaStoreOptions opts)
    throws RomaError
  {
    // Creates PutAdapter object
    PutAdapter pa = new PutAdapter()
    {
      // provide an implementation for the put() method.
      public void put( KxcContext ctx )
        throws RomaError
      {
        RomaContext.kxcOper( ctx.putToStorageQ( m_hStore ) );
      }
    };// end of anonymous inner class

    // this could throw a RomaError
    msg.marshalForStorage();

    m_ctx.putMsg( pa,
                  msg.m_Message,
                  opts.getBasePutOptions() );
  }

  /**
  * Retrieves a message from the storage queue according to
  * defaulted options.
  * <p>
  * Derived classes may override this method and return other,
  * more application-specific types of RomaStorable objects.
  *
  * @exception RomaError  thrown if retrieve operation fails.
  */
  public RomaStorable retrieve( )
    throws RomaError
  {
    RomaRetrieveOptions opts = new RomaRetrieveOptions( );
    return retrieve( opts );
  }

  /**
  * Retrieves a message from the storage queue according to the
  * supplied options.
  * <p>
  * Derived classes may override this method and return other,
  * more application-specific types of RomaStorable objects.
  *
  * @param  opt  retrieve options.
  *
  * @exception RomaError  thrown if retrieve operation fails.
  */
  public RomaStorable retrieve( RomaRetrieveOptions opt )
    throws RomaError
  {
    //----------------------------------------------------------------
    // Provide an implementation for the GetAdapter.get() method.
    //----------------------------------------------------------------
    GetAdapter ga = new GetAdapter()
    {
      public boolean get( KxcContext ctx )
        throws RomaError
      {
        RomaContext.kxcOper( ctx.getFromStorageQ( m_hStore ) );
        return true;
      }
    };// end of anonymous inner class 
    
    RomaMessage msg = new RomaMessage();
    
    if ( m_ctx.getMsg( ga, 
                       msg, 
                       opt.getBaseOptions() ) )
    {
      RomaStorable storable = new RomaStorable( msg );   
      return createStorable( storable );
    }

    return null;
  }
  
  /**
  * Retrieves a message from the storage queue according to the
  * supplied options into user defined buffer.
  * <p>
  * Derived classes may override this method and return other,
  * more application-specific types of RomaStorable objects.
  *
  * @param  opt  retrieve options.
  * @param  buffer buffer to receive data
  * @param  bufSize length of buffer,
  *                 actual data length on return. 
  *
  * @exception RomaError  thrown if retrieve operation fails.
  */
  public RomaStorable retrieve( RomaRetrieveOptions opt,
                                byte[]              buffer,
                                RomaInteger         bufSize )
    throws RomaError
  {
    //----------------------------------------------------------------
    // Provide an implementation for the GetAdapter.get() method.
    //----------------------------------------------------------------
    GetAdapter ga = new GetAdapter()
    {
      public boolean get( KxcContext ctx )
        throws RomaError
      {
        RomaContext.kxcOper( ctx.getFromStorageQ( m_hStore ) );
        return true;
      }
    };// end of anonymous inner class 

    RomaMessage msg = new RomaMessage();
    
    // copy the buffer size to protect
    RomaInteger bufLen = new RomaInteger( bufSize.intValue() );
    
    // Use buffer supplied instead of Roma auto-allocating
    msg.setMsgBuffer( buffer, bufLen );
    
    try {
      if ( m_ctx.getMsg( ga, 
                         msg, 
                         opt.getBaseOptions() ) )
      {
        RomaStorable storable = new RomaStorable( msg );   
        return createStorable( storable );
      }

      return null;
    }
    catch( RomaError apix ) {
      if ( apix.primaryInfo().returnCode() == Roma.RC_BUFFER_TOO_SMALL  ||
           apix.primaryInfo().returnCode() == Roma.RC_MESSAGE_SUPPRESSED )
      {
        // update buffer size to the required message length
        bufSize.setValue( bufLen.intValue() );
      }
      throw apix;
    }
  }
  
  /**
  * Check whether the storage queue is open.
  *
  * @return  true if still open, otherwise false
  */
  public boolean isOpen( )
  {
    try {
      RomaContext.kxcOper(m_ctx.kxcCtx().queryContext( ));
      return m_ctx.kxcCtx().queryContextReturn( );
    }
    catch (RomaError err) {
      ;
    }
    return false;
  }

  /**
  * Invoked when a message is successfully retrieved from the
  * storage queue, this method unmarshalls the recovered message.
  * <p>
  * It is intended that subclasses of RomaStorageQueue override
  * this method so that custom data unmarshalling can be achieved.
  * <p>
  * This implementation returns a RomaStorable object. containing
  *
  * @param msg  a newly-retrieved RomaStorable object.
  */
  protected RomaStorable createStorable( RomaStorable msg )
  {
    return new RomaStorable( msg );
  }
}
