//--------------------------------------------------------------------
//
// Copyright !Candle Corporation 1999-2003
//
//--------------------------------------------------------------------
//
// Module       : RomaSubscription.java
//
// Defines      : class RomaSubscription
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;

import com.candle.roma.kxc.raw.*;
import java.io.*;

/**
 * This class encapsulates subscription messages put by a Roma client.
 * Almost all of the methods provided are provided the same as the
 * RomaClientRequest class.
 * The class also provides the setOnRequest and setNewPubsOnly methods.
 * setOnRequest means that the subscriber won't get publications until it
 * requests them using RomaClient.RequestUpdates().
 * setNewPubsOnly means that the subscriber will only receive newly
 * issued publications and not retained publications.
 */

public class RomaSubscription
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 2001-2002";

  // RomaSubscription has a RomaClientRequest
  // Package visibility so other classes can access
  RomaClientRequest m_CR        = new RomaClientRequest("");
  RomaSubject       m_Sub       = new RomaSubject ("");
  boolean           m_onRequest = false;
  boolean           m_NewPubs   = false;

  /**
   * Constructs a subscription with default options and with the name
   * of a Roma business service and subject to subscribe to.
   *
   * @param svcName  Name of a Roma Business Service.
   * @param subject  Subject of the publication.
   */
  public RomaSubscription(String svcName, RomaSubject subject)
  {
    m_CR.setServiceName(svcName);
    setSubject(subject);
  }

  /**
   * Returns the name of the business service this subscription
   * is intended for. Uses package visibility - not public
  */
  String serviceName()
  {
    return m_CR.serviceName();
  }

  /**
   * Sets the name of the Roma Business Service that a publish 
   * operation will send this subscription to.
   *
   * @param svcName  Roma Business Service name.
  */
  public void setServiceName (String svcName)
  {
    m_CR.setServiceName (svcName);
  }

  /**
   * Associates an API profile with the operation.
   *
   * @param profile  the name of the API profile.
  */
  public void setApiProfileName (String profile)
  {
    m_CR.setApiProfileName (profile);
  }

  /**
   * Associates an error profile with the operation.
   *
   * @param profile  the name of the error profile.
  */
  public void setErrorProfileName (String profile)
  {
    m_CR.setErrorProfileName (profile);
  }

  /**
   * Specify if error profiling is to be suppressed
   *
   * @param b true to suppress error profiling
   */
   public void setSuppressErrorProfile (boolean b)
   {
     m_CR.setSuppressErrorProfile(b);
    }

  /**
   * Enables or disables the option to send the message to an
   * exception message service if the subscribe or unsubscribe
   * operation fails.
   *
   * @param select  one of :-
   * <ul>
   *    <li> true - <strong>do not</strong> perform a send to an
   *                exception message service if the operation fails.
   *    <li> false - allow a message to be sent to an exception
   *                 message service if the operation fails.
   * </ul>
  */
  public void setNoEms (boolean select)
  {
    m_CR.setNoEms (select);
  }

  /**
   * Specify if termination is supported
   *   
   * @param b true to indicate termination is supported.
   */
  public void setTermSupported (boolean b)
  { 
    m_CR.setTermSupported (b); 
  }

  /**
   * Specify whether a new theme should be allocated.
   * <p>
   * @param b  if true then a new theme is allocated.
  */
  public void setNewTheme(boolean b) 
  { 
    m_CR.setNewTheme(b); 
  }

  /**
   * Specify whether the theme should be retained for future operations.
   * <p>
   * @param b  if true then the theme is retained.
  */
  public void setRetainTheme(boolean b)
  { 
    m_CR.setRetainTheme(b); 
  }

  /**
   * Specify whether the theme should be closed automatically when the
   * operation completes.
   * <p>
   * @param b  if true then the theme will be closed.
  */
  public void setCloseTheme(boolean b)
  { 
    m_CR.setCloseTheme(b);
  }

  /**
   * Specify whether an existing theme should be used.
   * <p>
   * @param b  if true then the existing theme is used.
  */
  public void setUseTheme(boolean b)
  { 
    m_CR.setUseTheme(b); 
  }

  /**
   *  set or clear confirmation of delivery flag 
   */
  public void setCOD (boolean flag)
  {
    m_CR.setCOD (flag);
  }

  /**
   *  set or clear confirmation of arrival flag 
   */
  public void setCOA (boolean flag)
  {
    m_CR.setCOA (flag);
  }
  
  /**
   * Sets the datagram flag.
   * Controls whether or not confirmation messages are to be
   * returned.
   * <p>
   * If an API profile is to be used  (selected by invoking the
   * useApiProfile() method and verifiable by querying the
   * isApiProfileUsed() method) then this setting will be
   * overridden either by the profile or, if not available, the
   * default for this option.
   *
   * @param b  one of :-
   * <ul>
   *   <li>true - send DATAGRAM (and don't return confirmations)
   *   <li>false - send REQUEST (and return confirmations)
   * </ul>
  */
  public void setDatagram(boolean b)
  { 
    m_CR.setDatagram(b); 
  }
  
  /**
   * Set the message priority.
   * @param priority  this can be one of the following :-
   * <ul>
   * <li>PRIORITY_DEFAULT - use the default priority
   *     from the transport.
   * <li>PRIORITY_PROFILE - use the priority as specified in the 
   *     associated Put API profile. If no such profile or attribute,
   *     use PRIORITY_DEFAULT.
   * <li>a specific integer meaningful to the underlying transport
   *     e.g. 0..9 for MQSeries and BEAMQ,  0..7 for MSMQ 
   * </ul>
   */
  public void setPriority (int priority)
  {
    m_CR.setPriority(priority);
  }

  /**
   * Set a new value for the message perisitence.
   * @param  persistence  this can be one of the following :-
   *   <ul>
   *   <li>PERSISTENCE_YES
   *   <li>PERSISTENCE_NO
   *   <li>PERSISTENCE_PROFILE - use the persistence value as
   *       specified in the associated Put API profile. If no such
   *       profile or attribute, use PERSISTENCE_NO.
   *   </ul>
   */
  public void setPersistence (int persistence)
  {
    m_CR.setPersistence(persistence);
  }

  /**
   * Sets a new message feedback code.
   * @param feedback  new message feedback value.
   */
  public void setFeedback (int feedback)
  {
    m_CR.setFeedback(feedback);
  }

  /**
   * Set a new message expiry time.
   * @param expiry  the new expiry time value
   *   <strong>in seconds</strong> or EXPIRY_NEVER or EXPIRY_PROFILE.
   *   If EXPIRY_PROFILE is specified then
   *   the expiry value is retrieved from the associated Put API
   *   profile. If the profile is not available or else has no expiry
   *   attribute set, the EXPIRY_NEVER constant is used.
   */
  public void setExpiry (int expiry)
  {
    m_CR.setExpiry(expiry);
  }

  /**
   * Update the message theme.
   *
   * @param theme RomaTheme object.
   */
  public void setTheme (RomaTheme theme)
  {
    m_CR.setTheme (theme);
  }

  /**
   * Return the message theme. Theme can be set internally when
   * the message is being sent or received or else explicitly by
   * the user.
   */
  public RomaTheme theme()
  {
    return m_CR.theme();
  }

  /**
   * Set the name of this message
   * @param messageName name to be used
   */
  public void setMsgName (String messageName)
  {
    m_CR.setMsgName(messageName);
  }

  /**
   * Set security definition
   *
   * @param secDef security definition to be sent with message
   *
   */
  public void setSecurityDef (RomaSecurityDef secDef)
  {
    m_CR.setSecurityDef (secDef);
  }

  /**
   * Specify sync point definitiion
   * <p>
   * @param  sync definition to be used.
  */
  public void setSyncpointDef (RomaSyncpointDef sync)
  {
    m_CR.setSyncpointDef (sync);
  }

  /**
   * Set outgoing ILS data
   *
   * @param  ilsData The ILS data to be sent with message
   */
  public void setILSData(RomaILSData ilsData)
  {
    m_CR.setILSData(ilsData);
  }

  /**
   * Get incoming ILS data
   *
   */
  public RomaILSData getILSData()
  {
    return m_CR.getILSData();
  }

  /**
   * Set a subject of the subscription
   *
   * @param subject subject of the subscription
   */
  public void setSubject(RomaSubject subject)
  {
    m_Sub.setSubject(subject.subject());
    m_Sub.setSubjectStyle(subject.style());
  }

  /**
   * Sets the New Publications Only flag, which means that the
   * subscriber won't receive publications that were published
   * and retained before receipt of the subscription.
   *
   * @param b  one of :-
   * <ul>
   *   <li>true - the subscriber doesn't want retained publications
   *   <li>false - the subscriber does want retained publications
   * </ul>
  */
  public void setNewPubsOnly(boolean b)
  {
    m_NewPubs = b;
  }

  /**
   * Sets the On Request option, whereby the subscriber has to
   * request updates.
   * @see RomaClient#requestUpdates(RomaSubscription)
   *
   * @param b  one of :-
   * <ul>
   *   <li>true - the subscriber wants to request publications
   *   <li>false - the wants to receive publications as soon as they are available
   * </ul>
  */
  public void setOnRequest(boolean b)
  {
    m_onRequest = b;
  }
}

//--------------------------------------------------------------------
// End of file.
//--------------------------------------------------------------------

