//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcogto.hpp,v 1.9 2003/02/21 22:48:04 emueh Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcogto.hpp,v $
// $Revision: 1.9 $
//--------------------------------------------------------------------
// The RomaGetOptions public interface.
//--------------------------------------------------------------------

#ifndef __KXCOGTO_HPP
#define __KXCOGTO_HPP

#include <kxcocrq.hpp>

//--------------------------------------------------------------------
// The internal implementation class.
//--------------------------------------------------------------------
class RomaImplGetOptions;


//--------------------------------------------------------------------
// The RomaGetOptions class.
//
// This class encapsulates the criteria by which a Business Element
// retrieves a Client request or a Client retrieves a Business Service
// response.
//--------------------------------------------------------------------
class KXCODLL RomaGetOptions : public RomaTraceable
{
public:
    //----------------------------------------------------------------
    // Construct, using defaults.
    //----------------------------------------------------------------
    RomaGetOptions();

    //----------------------------------------------------------------
    // Construct, based on existing ClientRequest object
    //----------------------------------------------------------------
    RomaGetOptions(const RomaClientRequest& request);

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    virtual ~RomaGetOptions();

    //----------------------------------------------------------------
    // Copy and assignment.
    //----------------------------------------------------------------
    RomaGetOptions(const RomaGetOptions& getOptions);
    RomaGetOptions& operator = (const RomaGetOptions& getOptions);

    //----------------------------------------------------------------
    // Enables/disables getting native (non-Roma) messages.
    //
    // true - enable
    // false - disable
    //
    // If isApiProfileUsed() == true then either default or
    // profile value substitutes this option.
    //----------------------------------------------------------------
    void allowNative(bool flag = true);

    //----------------------------------------------------------------
    // Set and unset signal.
    //
    // MVS specific. See KXCMSG::Signal and KXCGOPT_USESIGNAL
    // in Roma documentation.
    //
    // If isApiProfileUsed() == true then either default or
    // profile value substitutes the KXCGOPT_USESIGNAL flag,
    // however not the signal number.
    //----------------------------------------------------------------
    void setSignal(KXCULONG signal);
    void unsetSignal();

    //----------------------------------------------------------------
    // The Application Management inline service can return an
    // indication that the application is no longer needed.  (e.g. if
    // the workload has decreased such that some instances of the
    // application are no longer needed.) Only applies to servers.
    //
    // This method allows a Business Element to register to receive
    // these indications (which can still be ignored).
    //----------------------------------------------------------------
    void setTermSupported(bool termAllowed = true);

    //----------------------------------------------------------------
    // This method allows an application to prevent the intervention
    // of any Error Analysis Routines defined in the configuration.
    //----------------------------------------------------------------
    void setSuppressErrorProfile(bool suppress = true);

    //----------------------------------------------------------------
    // Specify some data that Roma will pass onto a specified inline
    // service whenever Roma calls that service.
    //----------------------------------------------------------------
    void setILSData(const RomaILSData& data);

    //----------------------------------------------------------------
    // Sets security definition.
    //
    // Subsequent put() and get methods will pass specified security
    // information to the Roma inline services.
    //----------------------------------------------------------------
    void setSecurityDef(const RomaSecurityDef& securityDef);

    //----------------------------------------------------------------
    // Sets API profile name.
    //----------------------------------------------------------------
    void setApiProfileName(const char* name);

    //----------------------------------------------------------------
    // Sets error profile name.
    //----------------------------------------------------------------
    void setErrorProfileName(const char* name);

    //----------------------------------------------------------------
    // Define the get operation, as one of:
    //    GET_FIRST  - get first message  (destructive read)
    //    PEEK_FIRST - peek first message (non-destructive)
    //    PEEK_NEXT  - peek next message
    //    PEEK_SAME  - peek message under current read position
    //    GET_PEEKED - get message under current read position
    //
    // All operations work with messages which match the selector.
    // See setSelector() for selector option.
    //
    // Can be overridden by API profile.
    //----------------------------------------------------------------
    void setOperation(Roma::Operation operation);

    //----------------------------------------------------------------
    // Define the selection criteria for the get operation, as one of:
    //   BY_NONE     - get any message
    //   BY_MSGID    - match on MessageId, as specified by setMsgId()
    //   BY_CORRELID - match on CorrelationId
    //   BY_MSGID_AND_CORRELID - BY_MSGID + BY_CORRELID
    //   BY_MSGID_AND_THEME    - BY_MSGID + BY_THEME
    //
    // Can be overridden by API profile.
    //----------------------------------------------------------------
    void setSelector(Roma::GetSelector selector);

    //----------------------------------------------------------------
    // Define how theme is managed, as one of:
    //   THEME_RETAIN - retain theme after operation
    //   THEME_CLOSE  - close theme after operation
    //   THEME_DEFAULT_CONTROL -
    //      RomaClient::get()                   will close theme
    //      RomaBusinessElement::getException() will close theme
    //      RomaBusinessElement::get()          will retain theme
    //                                          (unless datagram)
    // Can be overridden by API profile.
    //----------------------------------------------------------------

    //----------------------------------------------------------------
    // Specify how theme is handled after the retrieve operation
    //----------------------------------------------------------------
    void setCloseTheme(bool f = true);
    void setRetainTheme(bool f = true);
    void setUseTheme(bool f = true);
    void setAnyTheme(bool f = true);

    //----------------------------------------------------------------
    // Sets theme used as a selector.
    // See setSelector().
    //----------------------------------------------------------------
    void setTheme(const RomaTheme& theme);

    //----------------------------------------------------------------
    // Sets msg id used as a selector.
    // See setSelector().
    //----------------------------------------------------------------
    void setMsgId(const RomaMsgId& msgId);

    //----------------------------------------------------------------
    // Sets correlation id used as a selector.
    // See setSelector().
    //----------------------------------------------------------------
    void setCorrelId(const RomaMsgId& correlId);

    //----------------------------------------------------------------
    // Sets number of milliseconds to wait for the message.
    //----------------------------------------------------------------
    void setWaitInterval(int milliseconds);
    void setWaitIndefinite(bool f = true);

    //----------------------------------------------------------------
    // If true, Roma will not allow the option to send the message
    //----------------------------------------------------------------
    void setNoEms(bool flag = true);

    //----------------------------------------------------------------
    // If notification is allowed, Roma can send responses straight
    // back to the client, ignoring any further business components.
    // (API profile may override)
    // The following method allows the application to specify that
    // notification messages may NOT be sent back to the client.
    //----------------------------------------------------------------
    void setNoNotification(bool flag = true);

    //----------------------------------------------------------------
    // Sets syncpoint definition.
    //----------------------------------------------------------------
    void setSyncpointDef(const RomaSyncpointDef& def);

    //----------------------------------------------------------------
    // Set the style for any received publications.
    //----------------------------------------------------------------
    void setSubjectStyle(Roma::Style style = Roma::STYLE_ROMA);


    //-----------------------------------------------------------------
    // IMPLEMENTATION METHODS.
    //-----------------------------------------------------------------
    virtual void trace(RomaTraceStream& stream) const;


protected:
    //-----------------------------------------------------------------
    // Pointer to the internal implementation.
    //-----------------------------------------------------------------
    RomaImplGetOptions* m_impl;

    friend class RomaImplProxy;
};


#endif // __KXCOGTO_HPP

