//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : kxcotb.hpp
//
// Declares     : RomaTypeBase & supplementary macros
//
// Description  : The RomaTypeBase class defines an interface for
//                run-time type checking and downcasting.
//
//                The following supplementary macros simplify
//                definition of the derived classes and run-time type
//                checking and downcasting:
//
//                ROMA_SAFE_PTR_CAST - casts pointer to the requested
//                                     type
//
//                ROMA_SAFE_CONST_PTR_CAST - casts const pointer to
//                                           the requested type
//
//                ROMA_IS_INSTANCEOF - checks if passed pointer can be
//                                     cast to the specified type
//
//                ROMA_TYPE_NAME - returns name of the class, which
//                                 instantiated the object
//
//                ROMA_DECLARE_TYPEID - used in the class declaration
//                                      to declare the run-time type
//                                      support
//
//                ROMA_DEFINE_TYPEIDx - used in the class's module to
//                                      define the run-time type
//                                      support for the class
//
//--------------------------------------------------------------------
// $Id: kxcotb.hpp,v 1.7 2003/02/21 22:48:09 emueh Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcotb.hpp,v $
// $Revision: 1.7 $
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// Protect against multiple includes.
//--------------------------------------------------------------------
#ifndef __KXCOTB_HPP
#define __KXCOTB_HPP

#include <kxco.hpp>

//--------------------------------------------------------------------
// Casts pointer to the specified type.
//
// Cast is done at run time so downcasts are safe.
//
// Parameters:
//   ptr - cast pointer, should be of RomaTypeBase* type or derived
//   type - name of class to cast to, class should declare and define
//     RTTI otherwise cast always will return 0.
//     See ROMA_DECLARE_TYPEID().
//
// Returns:
//   non 0 pointer of type "type*" - success
//   0 pointer of type "type*" - passed pointer was 0 or can't be
//     cast to the specified type
//--------------------------------------------------------------------
#define ROMA_SAFE_PTR_CAST( ptr, type ) \
    ((type*)(ptr?(ptr)->castTo(#type):(void*)0))

//--------------------------------------------------------------------
// Same as ROMA_SAFE_PTR_CAST() but accepts and returns const
// pointer.
//--------------------------------------------------------------------
#define ROMA_SAFE_CONST_PTR_CAST( ptr, type ) \
    ((const type*)(ptr?((ptr)->constCastTo(#type)):\
    (void*)0))

//--------------------------------------------------------------------
// Returns true if pointer can be cast to specified type.
//
// It just uses ROMA_SAFE_CONST_PTR_CAST() and checks returned ptr.
//--------------------------------------------------------------------
#define ROMA_IS_INSTANCEOF( ptr, type ) \
    (ROMA_SAFE_CONST_PTR_CAST( ptr, type ) != 0)

//--------------------------------------------------------------------
// Returns name of the object's class.
//--------------------------------------------------------------------
#define ROMA_TYPE_NAME( ptr ) \
    (ptr?ptr->typeId():"NULL")

//--------------------------------------------------------------------
// Declares RTTI for class.
//
// Should be placed in public part of class definition.
//
// For example:
//   myclass.hpp:
//
//     class MyClass : public RomaTypeBase
//     {
//     public:
//       ROMA_DECLARE_TYPEID();
//     };
//
// If ROMA_DECLARE_TYPEID() is in class definition then
// ROMA_DEFINE_TYPEIDx() should be placed in class's module outside
// any methods.
//
// For example:
//   myclass.cpp
//
//     ROMA_DEFINE_TYPEID(MyClass)
//--------------------------------------------------------------------
#define ROMA_DECLARE_TYPEID() \
    static RomaTypeId classTypeId(); \
    virtual RomaTypeId typeId() const; \
    virtual void* castTo( RomaTypeId )

//--------------------------------------------------------------------
// Defines RTTI for class.
//
// This macro should be used for classes directly derived from
// RomaTypeBase.
//--------------------------------------------------------------------
#define ROMA_DEFINE_TYPEID( Class ) \
    ROMA_DEFINE_TYPEID_( Class ) \
    void* Class::castTo( RomaTypeId type ) \
    {  \
        if( compareTypeId(classTypeId(),type) ) \
            return this; \
        void* ptr; \
        ROMA_CHECK_TYPEID_BASE( RomaTypeBase ) \
        return 0; \
    }

//--------------------------------------------------------------------
// Defines RTTI for class.
//
// This macro should be used for classes derived from single
// base. The base in turn, should be directly or indirectly derived
// from RomaTypeBase.
//--------------------------------------------------------------------
#define ROMA_DEFINE_TYPEID1( Class, Base1 ) \
    ROMA_DEFINE_TYPEID_( Class ) \
    void* Class::castTo( RomaTypeId type ) \
    {  \
        if( compareTypeId(classTypeId(),type) ) \
            return this; \
        void* ptr; \
        ROMA_CHECK_TYPEID_BASE( Base1 ) \
        return 0; \
    }

// implementation macro, not for use
#define ROMA_DEFINE_TYPEID_( Class )  \
    RomaTypeId Class::classTypeId() { return #Class; } \
    RomaTypeId Class::typeId() const { return classTypeId(); }

// implementation macro, not for use
#define ROMA_CHECK_TYPEID_BASE( base ) \
    if( (ptr=base::castTo(type)) != 0 ) \
        return ptr;

// type identifier
typedef const char* RomaTypeId;

//--------------------------------------------------------------------
// Class: RomaTypeBase
//
// Base for the classes which supports RTTI.
//
// This class is rarely to be used directly.
// The following macros intended to work with it:
//   ROMA_DECLARE_TYPEID()
//   ROMA_DEFINE_TYPEIDn()
//   ROMA_SAFE_PTR_CAST()
//   ROMA_SAFE_CONST_PTR_CAST()
//--------------------------------------------------------------------
class KXCODLL RomaTypeBase
{
public:
    // returns id of the class
    static RomaTypeId classTypeId();
    // returns id of the object's class
    virtual RomaTypeId typeId() const = 0;
    // casts object to different class
    virtual void* castTo( RomaTypeId id );
    // the same as castTo(), but returns const pointer
    const void* constCastTo( RomaTypeId id ) const;
    // compares two type identifiers
    static bool compareTypeId( RomaTypeId id1, RomaTypeId id2 );
};


//--------------------------------------------------------------------
// RomaAllocator class.
//
// Overrides 'new' and 'delete' operators for heap memory ownership.
//--------------------------------------------------------------------
class KXCODLL RomaAllocator
{
public:
    void* operator new(size_t);
    void operator delete(void*);
};


//--------------------------------------------------------------------
// End of module kxcotb.hpp
//--------------------------------------------------------------------
#endif

